require('../core/init');
const { expect } = require('chai');
const ApplicationHandler = require('../core/handler/application_handler');
const VersionHandler = require('../core/handler/bitsensor_version_handler');
const EnvironmentHandler = require('../core/handler/environment_handler');
const TimeHandler = require('../core/handler/time_handler');
const { Datapoint } = require('./../index');

describe('Post handle', () => {
  describe('Integration', () => {
    it('should not be empty', () => {
      const datapoint = new Datapoint();
      bitSensor.postHandlers[1].handle(datapoint);

      expect(datapoint.endpoint).to.not.empty;
    });
    it('should not be empty', () => {
      const datapoint = new Datapoint();
      bitSensor.postHandlers[0].handle(datapoint);
      expect(datapoint.meta).to.not.empty;
    });
  });
  describe('Unit', () => {
    describe('Version handler', () => {
      it('should not be empty', () => {
        bitSensor.postHandlers = [];
        bitSensor.addPostHandler(new VersionHandler());
        expect(bitSensor.postHandlers.length).to.be.above(0);
      });
      it('should use correct version', () => {
        const { version } = require('../package.json');
        const BitSensorVersion = require('../index').VERSION;
        expect(BitSensorVersion).to.be.equal(version);
      });
    });
    describe('Time handler', () => {
      it('should not be empty', () => {
        bitSensor.postHandlers = [];
        bitSensor.addPostHandler(new TimeHandler());
        expect(bitSensor.postHandlers.length).to.be.above(0);
      });
    });
    describe('Environment handler', () => {
      it('should not be empty', () => {
        bitSensor.postHandlers = [];
        bitSensor.addPostHandler(new EnvironmentHandler());
        expect(bitSensor.postHandlers.length).to.be.above(0);
      });
    });
    describe('Application handler', () => {
      it('should not be empty', () => {
        bitSensor.postHandlers = [];
        bitSensor.addPostHandler(new ApplicationHandler());
        expect(bitSensor.postHandlers.length).to.be.above(0);
      });
    });
  });
});
