const { expect } = require('chai');
require('../core/init');

//bitSensor.setConfig(require('./config.json'));

const bind = (fct) => {
  bitSensor.session.bind(fct);
};

describe('BitSensor', () => {
  describe('#initialized', () => {
    bitSensor.runSession(() => {
      it('must not be initialized initially', () => {
        expect(bitSensor.isInitialized()).to.be.false;
      });
    });
  });
  describe('#isDisabled', () => {
    bitSensor.runSession(() => {
      it('must be undefined initially', () => {
        expect(bitSensor.getDisabled()).to.be.undefined;
      });
      it('must equal to 1 when call setDisabled()', bind(() => {
        bitSensor.setDisabled();
        expect(bitSensor.getDisabled()).to.equal(1);
      }));
      it('must be disabled', bind(() => {
        expect(bitSensor.hasBeenDisabled()).to.be.true;
      }));
      it('must equal to 0 when call setEnabled()', bind(() => {
        bitSensor.setEnabled();
        expect(bitSensor.getDisabled()).to.equal(0);
      }));
      it('must be enabled', bind(() => {
        expect(bitSensor.hasBeenDisabled()).to.be.false;
      }));
    });
  });
  describe('#isHandlingRequest', () => {
    it('must be undefined initially', () => {
      expect(bitSensor.getHandlingRequest()).to.be.undefined;
    });
    it('must change when start and stop tracing', () => {
      bitSensor.runSession(() => {
        expect(bitSensor.getHandlingRequest()).to.equal(1);
        bitSensor.runSession(() => {
          expect(bitSensor.getHandlingRequest()).to.equal(2);
        });
        expect(bitSensor.getHandlingRequest()).to.equal(1);
      });
    });
  });
  describe('#clear', () => {
    it('must be set initially', () => {
      bitSensor.runSession(() => {
        expect(bitSensor.getDatapoint()).to.not.be.undefined;
        expect(bitSensor.getHandlingRequest()).to.not.be.undefined;
        expect(bitSensor.getDisabled()).to.not.be.undefined;
        bitSensor.clear();
        expect(bitSensor.getDatapoint()).to.be.null;
        expect(bitSensor.getHandlingRequest()).to.be.null;
        expect(bitSensor.getDisabled()).to.be.null;
      });
    });
  });
});
